import pandas
import numpy

from pandas import DataFrame
from pyfrbus.load_data import load_data

from sim_setup import denton_boot

def dentonizer(data: DataFrame):
    out = pandas.DataFrame()

    for col in data.columns:
        out.loc[:, col] = denton_boot(data.loc[:,col].to_numpy())

    out.index = pandas.period_range(start=str(data.index[0])+"Q1", end = str(data.index[len(data)-1])+"Q4", freq="Q")

    return(out)

def calc_delta(base: DataFrame, sim: DataFrame):
    delta = pandas.DataFrame()

    for col in base.columns:
        delta.loc[:, col] = sim.loc[:, col] - base.loc[:, col]
    
    return(delta)


import pandas
import scipy
import os

from pandas import DataFrame, Period, PeriodIndex, read_csv
from numpy import array, NaN
from typing import Union
from pyfrbus.load_data import load_data
from pyfrbus.frbus import Frbus
from punchcard import parse_tax_sim, read_macro, parse_corp_mtr, get_housing_subsidy_rates, nipa_scalar
from computation import denton_boot

def levers(data: DataFrame, card: DataFrame, run: int):
    #---------------------------------------------------------------------
    # Sets fiscal and monetary policy levers for all simulations.
    # Parameters:
    #   data (DataFrame): Raw longbase file from FOMC
    #   dfp  (str)      : Active fiscal policy setting (Debt targetting, surplus targetting, or exogenous)
    #   dmp  (str)      : Active monetary policy setting (Typically inertial Taylor rule)
    # Returns:
    #   data (DataFrame): Longbase with set levers
    #---------------------------------------------------------------------
    start = pandas.Period(card.loc[run, "start"], freq="Q")
    end = pandas.Period(card.loc[run, "end"], freq="Q")
    dfp = card.loc[run, "dfp"]
    dmp = card.loc[run, "dmp"]
    rstar = card.loc[run, "rstar"]
    
    fiscal = [col for col in data.columns if 'dfp' in col]
    fiscal.remove(dfp)
    monetary = [col for col in data.columns if 'dmp' in col]
    monetary.remove(dmp)

    data.loc[:, [dfp, dmp]] = 1
    data.loc[:, fiscal + monetary] = 0
    
    if(rstar=="on"):
        data.loc[:, "drstar"] = 1
    elif(rstar=="delay"):
        data.loc[start:start+39, "drstar"] = 0
        data.loc[start+40:end, "drstar"] = 1
    else:
        data.loc[:, "drstar"] = 0

    return(data)

def build_data(card: DataFrame, run: int, card_dates = False):
    #---------------------------------------------------------------------
    # This function constructs a baseline dataset using the mcontrol protocol
    # against which alternate scenario runs are compared. 
    # Parameters:
    #   card   (DataFrame): Punchcard of scenario specific parameters
    #   run          (int): Row for the card dataframe.
    #   card_dates  (bool): Flag for if the baseline is constructed by YBL or the FOMC
    # Returns:
    #   longbase (DataFrame): FRB longbase.txt file adjusted to suit this 
    #                         specific policy test.
    #---------------------------------------------------------------------
    longbase = load_data(os.path.join("/gpfs/gibbs/project/sarin/shared/raw_data/FRBUS", str(card.loc[run, "lb_version"]), str(card.loc[run, "lb_vintage"]), "LONGBASE.TXT"))
    
    # Get tax variable paths from Tax Simulator
    ts = parse_tax_sim(card, run, True)

    if card_dates:
        start = pandas.Period(card.loc[run, "start"][0:4], freq="Y")
        end = pandas.Period(card.loc[run, "end"][0:4], freq="Y")
    else:
        start = ts.index[0]
        end = ts.index[len(ts)-1]

    # Restrict to dates under consideration
    ts = ts.loc[start:end,:]

    # Read in macro economic forecast, combine with variable paths and convert to % of GDP
    macro = read_macro(card.loc[run, "macro_path"])
    macro = macro.loc[start:end,:]
    macro['TPN_ts'] = ts["iit_etr"] / macro["gdp"]
    macro['TCIN_cs'] = ts["revenues_corp_tax"] / macro["gdp"]
    macro['gfsrpn_macro'] = (macro["rev"] - macro["outlays"]) / macro["gdp"]

    # Calculate / Parse MTRs
    per_mtr = get_housing_subsidy_rates(card, run)
    per_mtr = per_mtr.loc[:,"base"]
    
    start = start.asfreq('Q') - 3
    end = end.asfreq('Q')

    # Convert variables to "FRBUS $"
    temp = longbase.loc[start:end, "xgdpn"]
    temp = temp.groupby(temp.index.year).sum() 
    temp.index = macro.index
    macro["TPN_ts"] *= temp
    macro["TCIN_cs"] *= temp 
    macro["gfsrpn_macro"] *= temp

    # Interpolate annual values to quarterly
    TPN_fs = denton_boot(macro["TPN_ts"].to_numpy())
    TCIN_fs = denton_boot(macro["TCIN_cs"].to_numpy())
    gfsrpn_dent = denton_boot(macro["gfsrpn_macro"].to_numpy())
    
    # Set up fiscal/monetary policy levers
    frbus = Frbus("/gpfs/gibbs/project/sarin/shared/conda_pkgs/pyfrbus/models/model.xml", mce="mcap+wp")

    longbase = levers(longbase, card, run)

    # Set paths and solve
    with_adds = frbus.init_trac(start, end, longbase)
    
    with_adds.loc[start:end, "tpn_t"] = TPN_fs * nipa_scalar(card, run)
    with_adds.loc[start:end, "tcin_t"] = TCIN_fs
    with_adds.loc[start:end, "trfpm"] = per_mtr
    with_adds.loc[start:end, "trfcim"] = parse_corp_mtr(card, run)
    with_adds.loc[start:, "gfdrt"] = 1.1 #with_adds.loc[start,"gfdbtn"] / with_adds.loc[start,"xgdpn"] # Average? Max? Hardcode?
    with_adds.loc[start:end, "gfsrpn_t"] = gfsrpn_dent

    with_adds.loc[start:end, "xgdpn_t"] = with_adds.loc[start:end, "xgdpn"]
    with_adds.loc[start:end, "xgdp_t"] = with_adds.loc[start:end, "xgdp"]
    with_adds.loc[start:end, "picxfe_t"] = with_adds.loc[start:end, "picxfe"]
    with_adds.loc[start:end, "lur_t"] = with_adds.loc[start:end, "lur"]
    with_adds.loc[start:end, "rff_t"] = with_adds.loc[start:end, "rff"]

    sim = frbus.mcontrol(start, end, with_adds, 
        targ=["tpn",      "tcin",      "xgdpn",      "gfsrpn",      "xgdp",      "picxfe",      "lur",      "rff"], 
        traj=["tpn_t",    "tcin_t",    "xgdpn_t",    "gfsrpn_t",    "xgdp_t",    "picxfe_t",    "lur_t",    "rff_t"], 
        inst=["trp_aerr", "trci_aerr", "xgdpn_aerr", "gfsrpn_aerr", "xgdp_aerr", "picxfe_aerr", "lur_aerr", "rff_aerr"])
    
    # Filter out values not found in original longbase (they all contain '_')
    out = sim.filter(regex="^((?!_).)*$")

    # Replace section of original longbase within our timeframe with new values
    longbase.loc[start:end,:] = out.loc[start:end,:]

    return(longbase)

def calc_tpn_path(card: DataFrame, run: int, data: DataFrame, card_dates = False):
    #---------------------------------------------------------------------
    # Calculates Personal Income Tax Revenue path based on Tax Simulator output
    # Parameters:
    #   card     (DataFrame): Punchcard of scenario specific parameters
    #   run            (int): Row for the card dataframe.
    #   data     (DataFrame): Processed dataset pre-scenario simulation
    #   card_dates    (bool): (optional) Flag to derive scenario start and end dates from the card
    # Returns:
    #   TPN_fs (Vector[int]): Quarterly Personal Income Tax Revenue in FRBUS dollars
    #---------------------------------------------------------------------    
    ts = parse_tax_sim(card, run, card.loc[run, "ID"]=="baseline")
    
    if card_dates:
        start = pandas.Period(card.loc[run, "start"][0:4], freq="Y")
        end = pandas.Period(card.loc[run, "end"][0:4], freq="Y")
    else:
        start = ts.index[0]
        end = ts.index[len(ts)-1]

    macro = read_macro(card.loc[run, "macro_path"])
    macro = macro.loc[start:end]
    macro['TPN_ts'] = ts["iit_etr"] / macro["gdp"]
    
    start = start.asfreq('Q') - 3
    end = end.asfreq('Q')

    temp = data.loc[start:end, "xgdpn"]
    temp = temp.groupby(temp.index.year).sum() 
    temp.index = macro.index
    macro["TPN_ts"] *= temp

    TPN_fs = (denton_boot(macro["TPN_ts"].to_numpy()))
    TPN_fs *= nipa_scalar(card, run)
    
    return(TPN_fs) 

def calc_tcin_path(card: DataFrame, run: int, data: DataFrame, card_dates = False):
    #---------------------------------------------------------------------
    # Calculates Corporate Income Tax Revenue path based on Tax Simulator output
    # Parameters:
    #   card     (DataFrame): Punchcard of scenario specific parameters
    #   run            (int): Row for the card dataframe.
    #   data     (DataFrame): Processed dataset pre-scenario simulation
    #   card_dates    (bool): (optional) Flag to derive scenario start and end dates from the card
    # Returns:
    #   TPN_fs (Vector[int]): Quarterly Corporate Income Tax Revenue in FRBUS dollars
    #---------------------------------------------------------------------    
    if(card.loc[run, "ID"]=="baseline"):
        ts = parse_tax_sim(card, run, True)
    else:
        ts = parse_tax_sim(card, run, False)

    if card_dates:
        start = pandas.Period(card.loc[run, "start"][0:4], freq="Y")
        end = pandas.Period(card.loc[run, "end"][0:4], freq="Y")
    else:
        start = ts.index[0]
        end = ts.index[len(ts)-1]

    macro = read_macro(card.loc[run, "macro_path"])
    macro = macro.loc[start:end]
    ts = ts.loc[start:end]
    macro['TCIN_cs'] = ts["revenues_corp_tax"] / macro["gdp"]

    start = start.asfreq('Q') - 3
    end = end.asfreq('Q')

    temp = data.loc[start:end, "xgdpn"]
    temp = temp.groupby(temp.index.year).sum() 
    temp.index = macro.index
    macro["TCIN_cs"] *= temp

    TCIN_fs = (denton_boot(macro["TCIN_cs"].to_numpy()))
    
    return(TCIN_fs)


def dynamic_rev(card: DataFrame, run: int, start: Period, end: Period, data: DataFrame, frbus: Frbus, outpath = None, delta=False):
    #---------------------------------------------------------------------
    # Calculates Personal Income Tax Revenue path based on Tax Simulator output
    # Parameters:
    #   card     (DataFrame): Punchcard of scenario specific parameters
    #   run            (int): Row for the card dataframe.
    #   start       (Period): Beginning date of scenario
    #   end         (Period): End date of scenario
    #   data     (DataFrame): Processed dataset pre-scenario simulation
    #   frbus        (Frbus): Frbus model object
    #   outpath       (path): (optional) Filepath for adjusted longbase output
    #   delta         (bool): (optional) Flag for if dynamic revenue path includes an off model delta
    # Returns:
    #   dynamic  (DataFrame): Dynamic revenue score of scenario. Includes conventional score.
    #---------------------------------------------------------------------    
    macro = read_macro(card.loc[run, "macro_path"])
    macro = macro.loc[start.asfreq('Y'):end.asfreq('Y'), :]

    ts = parse_tax_sim(card, run, base=card.loc[run, "ID"]=="baseline")

    per_mtr = get_housing_subsidy_rates(card, run)
    per_mtr = per_mtr.loc[:,"scen"]

    if delta:
        data.loc[start:end, "trp_t"] = ((data.loc[start:end, "tpn_d"] + calc_tpn_path(card, run, data, True)) / (data.loc[start:end, "ypn"] - data.loc[start:end, "gtn"]))
        data.loc[start:end, "trci_t"] = (data.loc[start:end, "tcin_d"] + calc_tcin_path(card, run, data, True)) / data.loc[start:end, "ynicpn"]
    
    else:
        data.loc[start:end, "trp_t"] = (calc_tpn_path(card, run, data, True)) / (data.loc[start:end, "ypn"] - data.loc[start:end, "gtn"])
        data.loc[start:end, "trci_t"] = (calc_tcin_path(card, run, data, True)) / data.loc[start:end, "ynicpn"]
    
    data.loc[start:end, "trfpm"] = per_mtr
    data.loc[start:end, "trfcim"] = parse_corp_mtr(card, run)
    data.loc[start:, "gfdrt"] = 1.1 #data.loc[start,"gfdbtn"] / data.loc[start,"xgdpn"] # Average? Max? Hardcode?
    
    if card.loc[run, "ID"]=="baseline":
        macro['gfsrpn_macro'] = (macro["rev"] - macro["outlays"]) / macro["gdp"]
        temp = data.loc[start:end, "xgdpn"]
        temp = temp.groupby(temp.index.year).sum()
        macro['gfsrpn_macro'] *= temp
        gfsrpn_dent = denton_boot(macro['gfsrpn_macro'].to_numpy())

        data.loc[start:end, "xgdpn_t"]  = data.loc[start:end, "xgdpn"]
        data.loc[start:end, "xgdp_t"]   = data.loc[start:end, "xgdp"]
        data.loc[start:end, "picxfe_t"] = data.loc[start:end, "picxfe"]
        data.loc[start:end, "lur_t"]    = data.loc[start:end, "lur"]
        data.loc[start:end, "rff_t"]    = data.loc[start:end, "rff"]
        data.loc[start:end, "gfsrpn_t"] = gfsrpn_dent
        
        sim = frbus.mcontrol(start, end, data, 
            targ=["trp",      "trci",      "xgdpn",      "gfsrpn",      "xgdp",      "picxfe",      "lur",      "rff"], 
            traj=["trp_t",    "trci_t",    "xgdpn_t",    "gfsrpn_t",    "xgdp_t",    "picxfe_t",    "lur_t",    "rff_t"], 
            inst=["trp_aerr", "trci_aerr", "xgdpn_aerr", "gfsrpn_aerr", "xgdp_aerr", "picxfe_aerr", "lur_aerr", "rff_aerr"])

    else:
        sim = frbus.mcontrol(start, end, data, targ=["trp", "trci"], traj=["trp_t", "trci_t"], inst=["trp_aerr", "trci_aerr"])
    
    data_yr = data.groupby(data.index.year).sum()
    sim_yr  = sim.groupby(sim.index.year).sum()
    sim_yr_avg  = sim.groupby(sim.index.year).mean() 

    macro = macro.loc[start.asfreq('Y'):end.asfreq('Y'),]
    ts = ts.loc[start.asfreq('Y'):end.asfreq('Y'),]
    data_yr = data_yr.loc[start.year:end.year,]
    sim_yr = sim_yr.loc[start.year:end.year,]
    sim_yr_avg = sim_yr_avg.loc[start.year:end.year,]
    
    macro.index = data_yr.index

    dynamic = pandas.DataFrame()
    dynamic["TPN"] =  (sim_yr["tpn"] / nipa_scalar(card, run)) * (macro["gdp"]/data_yr["xgdpn"]) 
    dynamic["TCIN"] = sim_yr["tcin"] * (macro["gdp"]/data_yr["xgdpn"])
    dynamic["total_dynamic"] = dynamic["TPN"] + dynamic["TCIN"]

    dynamic.index = pandas.PeriodIndex(dynamic.index, freq = "Y")

    dynamic["iit_rev"] = ts["iit_etr"]
    dynamic["corp_rev"] = ts["revenues_corp_tax"]
    dynamic["total_conventional"] = dynamic["iit_rev"] + dynamic["corp_rev"]

    if outpath is not None:
        if not os.path.exists(outpath):
            os.makedirs(outpath)

        out = sim.filter(regex="^((?!_).)*$")
        
        longbase = load_data(os.path.join("/gpfs/gibbs/project/sarin/shared/raw_data/FRBUS", str(card.loc[run, "lb_version"]), str(card.loc[run, "lb_vintage"]), "LONGBASE.TXT"))
        longbase.loc[start:end,:] = out.loc[start:end,:]
        longbase.to_csv(os.path.join(outpath,card.loc[run,"ID"]+"_LONGBASE.csv"))

    return(dynamic)




import numpy as np
import pandas

from numpy import array
from pandas import DataFrame, Period, PeriodIndex

def denton_boot(Annual: array):
    #---------------------------------------------------------------------
    # This function takes in annual tax revenue data and interpolates it 
    # to quarterly frequency. This method smooths the new year step up.
    # Parameters:
    #   Annual (array): Annual tax revenue data of length T
    # Returns:
    #   out (array): Quarterly tax revenue data of length 4T
    #---------------------------------------------------------------------
    T = len(Annual)
    Tq = T * 4

    C = calc_c(Tq)
    J = calc_j(T)
    J_t = np.transpose(J * -1)
    zero4 = np.zeros((T,T))
    
    lhs1 = np.concatenate((C, J), axis=0)
    lhs2 = np.concatenate((J_t, zero4), axis=0)
    lhs = np.linalg.inv(np.concatenate((lhs1, lhs2), axis=1))   
    rhs = np.append(np.zeros(Tq), Annual)

    out = np.dot(lhs, rhs)
    return(out[0:Tq])

def calc_j(T: int):
    pattern = np.array([1,1,1,1])
    return(np.kron(np.eye(T), pattern))

def calc_c(Tq: int):
    #---------------------------------------------------------------------
    # This function creates a square band matrix for Denton interpolation
    # Parameters:
    #   Tq (int) : The number of quarters to be interpolated
    # Returns:
    #   C (numpy): A Tq X Tq band matrix
    #---------------------------------------------------------------------
    base = inner_band([2,-8,12,-8,2], Tq-4)
    v0 = np.zeros(Tq)
    v1 = np.zeros(Tq)
    np.put(v0, [0,1,2], [2,-4,2])
    np.put(v1, [0,1,2,3], [-4,10,-8,2])
    out = np.insert(base, 0, v1, axis=0)
    out = np.insert(out, 0, v0, axis=0)
    out = np.insert(out, len(out), np.flip(v1), axis=0)
    out = np.insert(out, len(out), np.flip(v0), axis=0)
    return(out)

def inner_band(a, W):
    # Thank you: http://scipy-lectures.org/advanced/advanced_numpy/#indexing-scheme-strides
    a = np.asarray(a)
    p = np.zeros(W-1,dtype=a.dtype)
    b = np.concatenate((p,a,p))
    s = b.strides[0]
    strided = np.lib.stride_tricks.as_strided
    return strided(b[W-1:], shape=(W,len(a)+W-1), strides=(-s,s))

def smooth_path(base: DataFrame, scen: DataFrame):
    weights_base = np.linspace(start = 0, stop = 1, num = len(base))
    weights_scen = np.flip(weights_base)
    out = (base * weights_base) + (scen * weights_scen)
    print(out)

    return(out)

# smooth_weight = NA
# smooth_weight.loc[:start_smooth] = 0
# smooth_weight.loc[start_smooth:end_smooth] = (date - start_smooth)/(end_smooth - start_smooth)
# smooth_weight.loc[end_smooth:] = 1

# rff_t = rff_baseline * smooth_weight + rff_scenario * (1-smooth_weight)


import os
import pandas
import numpy
import csv
import sys

from pandas import DataFrame, Period, PeriodIndex, read_csv
from pyfrbus.load_data import load_data
from pyfrbus.frbus import Frbus
from typing import Union

def parse_tax_sim(card: DataFrame, run: int, base = False) -> DataFrame:
    #---------------------------------------------------------------------
    # Reads in Tax Simulator's personal output and prepares it for computation.
    # Parameters:
    #   card   (DataFrame): Punchcard of scenario specific parameters
    #   run          (int): Row for the card dataframe.
    #   base        (bool): Flag for if the input is a baseline run
    # Returns:
    #   ts     (DataFrame): Tax Simulator output including Effective Tax Revenue
    #---------------------------------------------------------------------
    version = str(card.loc[run, "ts_version"])
    vintage = str(card.loc[run, "ts_vintage"])
    if(base):
        ID = "baseline/static"
    else:
        ID = str(card.loc[run, "ID"]) + "/conventional"

    ts = pandas.read_csv(os.path.join("/gpfs/gibbs/project/sarin/shared/model_data/Tax-Simulator", version, vintage, ID, "totals/receipts.csv"), index_col = 0)
    #ts = pandas.read_csv(os.path.join("/vast/palmer/scratch/sarin/jar335/Tax-Simulator", version, vintage, ID, "totals/receipts.csv"), index_col = 0)
    ts.index = pandas.PeriodIndex(ts.index, freq="Y")

    ts['iit_etr'] = ts.loc[:,'revenues_income_tax'] - ts.loc[:,'outlays_tax_credits'] + ts.loc[:,'revenues_estate_tax']
    
    return(ts)

def parse_corp_mtr(card: DataFrame, run: int) -> DataFrame:
    #---------------------------------------------------------------------
    # Reads in Tax Simulator's corporate output and prepares it for computation.
    # Parameters:
    #   card   (DataFrame): Punchcard of scenario specific parameters
    #   run          (int): Row for the card dataframe.
    # Returns:
    #   cs     (DataFrame): Tax Simulator output.
    #---------------------------------------------------------------------
    version = str(card.loc[run, "ts_version"])
    vintage = str(card.loc[run, "ts_vintage"])
    ID      = str(card.loc[run, "ID"])
    start = pandas.Period(card.loc[run, "start"], freq='Q')
    end = pandas.Period(card.loc[run, "end"], freq='Q')

    cs = pandas.read_csv(os.path.join("/gpfs/gibbs/project/sarin/shared/model_data/Tax-Simulator", version, vintage, ID, "static/supplemental/tax_law.csv"))
    cs = cs.loc[((cs["filing_status"]==1) & (cs["year"].between(start.year, end.year))), "corp.rate"]

    cs = numpy.repeat(cs, 4)
    cs.index = pandas.period_range(start = start, end = end)
    return(cs)

def read_macro(path: str):
    #---------------------------------------------------------------------
    # Reads in macroeconomic projections.
    # Parameters:
    #   path           (path): Location of 
    # Returns:
    #   macro     (DataFrame): Tax Simulator output including Effective Tax Revenue
    #---------------------------------------------------------------------
    base = pandas.read_csv(os.path.join(path, "historical.csv"), index_col=0)
    proj = pandas.read_csv(os.path.join(path, "projections.csv"), index_col=0)

    out = base.append(proj)
    out.index = pandas.PeriodIndex(out.index, freq="Y")
    
    return(out)

def get_housing_subsidy_rates(card: DataFrame, run: int):
    start = pandas.Period(card.loc[run, "start"], freq='Q')
    end = pandas.Period(card.loc[run, "end"], freq='Q')
    outroot = os.path.join('/gpfs/gibbs/project/sarin/shared/model_data/Tax-Simulator', str(card.loc[run, "ts_version"]), str(card.loc[run, "ts_vintage"]))
    out = DataFrame(columns=["base","scen"])

    for y in pandas.period_range(start.asfreq("Y"), end.asfreq("Y")):
        base = pandas.read_csv(os.path.join(outroot, "baseline/static/detail", str(y)+'.csv'))
        base = base.loc[:,["weight", "first_mort_int", "mtr_first_mort_int"]]

        scen = pandas.read_csv(os.path.join(outroot, str(card.loc[run, "ID"]), "static/detail", str(y)+'.csv'))
        base["scen"] = scen.loc[:,"mtr_first_mort_int"]
        
        add = [numpy.average(base.loc[:,"mtr_first_mort_int"], weights = (base.loc[:,"weight"] * base.loc[:,"first_mort_int"])),\
            numpy.average(base.loc[:,"scen"], weights = (base.loc[:,"weight"] * base.loc[:,"first_mort_int"]))]
        out.loc[len(out.index)] = add
    
    out = out.iloc[numpy.repeat(numpy.arange(len(out)),4)]
    out *= -100
    out.index = pandas.period_range(start = start, end = end)
    return(out)

def nipa_scalar(card: DataFrame, run: int):
    tots = load_data(os.path.join("/gpfs/gibbs/project/sarin/shared/raw_data/NIPA", str(card.loc[run, "nipa_vintage"]), "3-1.csv"))
    snl = load_data(os.path.join("/gpfs/gibbs/project/sarin/shared/raw_data/NIPA", str(card.loc[run, "nipa_vintage"]), "3-3.csv"))
    return(1 + numpy.mean(snl.loc[Period("2009Q2", freq="Q"):Period("2019Q4", freq="Q"), "personal-current-taxes"] / tots.loc[Period("2009Q2", freq="Q"):Period("2019Q4", freq="Q"),"personal-current-taxes"]))

def run_out(card: DataFrame, stamp: str, run: int, data: DataFrame, path = None, run_type="baseline"):
    # Create base path 

    if path is not None:
        path = os.path.join("/gpfs/gibbs/project/sarin/shared/model_data/FRBUS", stamp)
        
        if not os.path.exists(path):
            os.makedirs(path)

    if run_type == "baseline":
        data = data.filter(regex="^((?!_).)*$")
        
        longbase = load_data(os.path.join("/gpfs/gibbs/project/sarin/shared/raw_data/FRBUS", str(card.loc[run, "lb_version"]), str(card.loc[run, "lb_vintage"]), "LONGBASE.TXT"))
        longbase.loc[start:end,:] = data.loc[start:end,:]
        longbase.to_csv(os.path.join(path,card.loc[run,"ID"]+"_LONGBASE.csv"))
    
    return()




from setuptools import setup

setup(
  name="FRBUS",
  version = "1.0",
  author='Yale Budget Lab',  
  install_requires=[
        "pandas",
        "scipy",
        "numpy",
        "black",
        "flake8",
        "mypy",
        "typing_extensions",
        "multiprocess",
        "sympy==1.3",
        "symengine",
        "matplotlib",
        "lxml",
        "networkx",
        "scikit-umfpack==0.3.3",
        "pyfrbus",
  ],
  packages=["FRBUS"],
)
